@@ -5,8 +5,8 @@ if False:  # MYPY
 
 _package_data = dict(
     full_package_name='ruamel.yaml',
-    version_info=(0, 17, 1),
-    __version__='0.17.1',
+    version_info=(0, 17, 2),
+    __version__='0.17.2',
     author='Anthon van der Neut',
     author_email='a.van.der.neut@ruamel.eu',
     description='ruamel.yaml is a YAML parser/emitter that supports roundtrip preservation of comments, seq/map flow style, and map key order',  # NOQA
@@ -35,7 +35,8 @@ _package_data = dict(
         env='*f',  # f for 3.5
         fl8excl='_test/lib',
     ),
-    universal=True,
+    # universal=True,
+    python_requires='>=3',
     rtfd='yaml',
 )  # type: Dict[Any, Any]
 
